@@ -3,15 +3,15 @@
  */
 
 /* 
- * Copyright (C) 1986, 1988, 1989, 1991 the Free Software Foundation, Inc.
+ * Copyright (C) 1986, 1988, 1989, 1991, 1992 the Free Software Foundation, Inc.
  * 
  * This file is part of GAWK, the GNU implementation of the
  * AWK Progamming Language.
  * 
  * GAWK is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 1, or (at your option)
- * any later version.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  * 
  * GAWK is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,7 +20,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with GAWK; see the file COPYING.  If not, write to
- * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ * the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #include "awk.h"
@@ -103,7 +103,7 @@ hash(s, len)
 register char *s;
 register int len;
 {
-	register unsigned int h = 0, g;
+	register unsigned long h = 0, g;
 
 	while (len--) {
 		h = (h << 4) + *s++;
@@ -128,26 +128,18 @@ NODE *symbol;
 register NODE *subs;
 int hash1;
 {
-	register NODE *bucket;
-	int chained = 0;
+	register NODE *bucket, *prev = 0;
 
 	for (bucket = symbol->var_array[hash1]; bucket; bucket = bucket->ahnext) {
 		if (cmp_nodes(bucket->ahname, subs) == 0) {
-			if (chained) {	/* move found to front of chain */
-			register NODE *this, *prev;
-				for (prev = this = symbol->var_array[hash1];
-				     this; prev = this, this = this->ahnext) {
-					if (this == bucket) {
-						prev->ahnext = this->ahnext;
-						this->ahnext = symbol->var_array[hash1];
-						symbol->var_array[hash1] = this;
-					}
-				}
+			if (prev) {	/* move found to front of chain */
+				prev->ahnext = bucket->ahnext;
+				bucket->ahnext = symbol->var_array[hash1];
+				symbol->var_array[hash1] = bucket;
 			}
 			return bucket;
-		}
-		if (bucket)
-			chained = 1;
+		} else
+			prev = bucket;	/* save previous list entry */
 	}
 	return NULL;
 }
@@ -208,6 +200,10 @@ NODE *symbol, *subs;
 			return &(bucket->ahvalue);
 		}
 	}
+
+	/* It's not there, install it. */
+	if (do_lint && subs->stlen == 0)
+		warning("subscript of array is null string");
 	getnode(bucket);
 	bucket->type = Node_ahash;
 	bucket->ahname = dupnode(subs);
