@@ -3,15 +3,15 @@
  */
 
 /* 
- * Copyright (C) 1991, the Free Software Foundation, Inc.
+ * Copyright (C) 1991, 1992, the Free Software Foundation, Inc.
  * 
  * This file is part of GAWK, the GNU implementation of the
  * AWK Progamming Language.
  * 
  * GAWK is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 1, or (at your option)
- * any later version.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  * 
  * GAWK is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,7 +20,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with GAWK; see the file COPYING.  If not, write to
- * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ * the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #ifdef __STDC__
@@ -44,14 +44,14 @@ extern int strncmp P((const char *, const char *, int));
 #ifndef VMS
 extern char *strerror P((int));
 #else
-extern char *strerror();	/* extern char *strerror(int,...); */
+extern char *strerror P((int,...));
 #endif
 extern char *strchr P((const char *, int));
 extern char *strrchr P((const char *, int));
 extern char *strstr P((const char *s1, const char *s2));
 extern int strlen P((const char *));
 extern long strtol P((const char *, char **, int));
-#ifndef _MSC_VER
+#if !defined(_MSC_VER) && !defined(__GNU_LIBRARY__)
 extern int strftime P((char *, int, const char *, const struct tm *));
 #endif
 extern time_t time P((time_t *));
@@ -63,7 +63,7 @@ extern int memcmp P((const aptr_t, const aptr_t, size_t));
 
 /* extern int fprintf P((FILE *, char *, ...)); */
 extern int fprintf P(());
-#ifndef MSDOS
+#if !defined(MSDOS) && !defined(__GNU_LIBRARY__)
 extern int fwrite P((const char *, int, int, FILE *));
 extern int fputs P((const char *, FILE *));
 extern int unlink P((const char *));
@@ -88,12 +88,15 @@ extern double pow P((double x, double y));
 extern double atof P((char *));
 extern double strtod P((const char *, char **));
 extern int fstat P((int, struct stat *));
+extern int stat P((const char *, struct stat *));
 extern off_t lseek P((int, off_t, int));
 extern int fseek P((FILE *, long, int));
 extern int close P((int));
+extern int creat P(());
 extern int open P(());
 extern int pipe P((int *));
 extern int dup P((int));
+extern int dup2 P((int,int));
 extern int fork P(());
 extern int execl P((/* char *, char *, ... */));
 extern int read P((int, char *, int));
