@@ -3,15 +3,15 @@
  */
 
 /* 
- * Copyright (C) 1991 the Free Software Foundation, Inc.
+ * Copyright (C) 1991, 1992 the Free Software Foundation, Inc.
  * 
  * This file is part of GAWK, the GNU implementation of the
  * AWK Progamming Language.
  * 
  * GAWK is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 1, or (at your option)
- * any later version.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  * 
  * GAWK is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,7 +20,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with GAWK; see the file COPYING.  If not, write to
- * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ * the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #include "awk.h"
@@ -57,30 +57,33 @@ int dfa;
 }
 
 int
-research(rp, str, len, need_start)
+research(rp, str, start, len, need_start)
 Regexp *rp;
 register char *str;
+int start;
 register int len;
 int need_start;
 {
-	int count;
-	int try_backref;
-	char save1;
-	char save2;
-	char *ret = &save2;
+	char *ret = str;
 
 	if (rp->dfa) {
-		save1 = str[len];
-		str[len] = '\n';
-		save2 = str[len+1];
-		ret = regexecute(&(rp->dfareg), str, str+len+1, 0, &count,
-					&try_backref);
-		str[len] = save1;
-		str[len+1] = save2;
+		char save1;
+		char save2;
+		int count = 0;
+		int try_backref;
+
+		save1 = str[start+len];
+		str[start+len] = '\n';
+		save2 = str[start+len+1];
+		ret = regexecute(&(rp->dfareg), str+start, str+start+len+1, 1,
+					&count, &try_backref);
+		str[start+len] = save1;
+		str[start+len+1] = save2;
 	}
 	if (ret) {
 		if (need_start || rp->dfa == 0)
-			return re_search(&(rp->pat), str, len, 0, len, &(rp->regs));
+			return re_search(&(rp->pat), str, start+len, start,
+					len, &(rp->regs));
 		else
 			return 1;
 	 } else
@@ -94,12 +97,12 @@ Regexp *rp;
 	free(rp->pat.buffer);
 	free(rp->pat.fastmap);
 	if (rp->dfa)
-		regfree(&(rp->dfareg));
+		reg_free(&(rp->dfareg));
 	free(rp);
 }
 
 void
-regerror(s)
+reg_error(s)
 const char *s;
 {
 	fatal(s);
@@ -142,3 +145,10 @@ NODE *t;
 	t->re_flags |= IGNORECASE;
 	return t->re_reg;
 }
+
+void
+resetup()
+{
+	(void) re_set_syntax(RE_SYNTAX_AWK);
+	regsyntax(RE_SYNTAX_AWK, 0);
+}
