@@ -17,10 +17,6 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 
-#ifdef __GNUC__
-  #pragma once
-#endif
-
 #ifndef __REGEXP_LIBRARY
 #define __REGEXP_LIBRARY
 
@@ -38,7 +34,7 @@
 
 
 /* This defines the various regexp syntaxes.  */
-extern int obscure_syntax;
+extern long obscure_syntax;
 
 
 /* The following bits are used in the obscure_syntax variable to choose among
@@ -48,25 +44,25 @@ extern int obscure_syntax;
      parentheses are needed for literal searching.
    If not set, backslash-parentheses are grouping, and plain parentheses
      are for literal searching.  */
-#define RE_NO_BK_PARENS	1
+#define RE_NO_BK_PARENS	1L
 
 /* If this bit is set, plain | serves as the `or'-operator, and \| is a 
      literal.
    If not set, \| serves as the `or'-operator, and | is a literal.  */
-#define RE_NO_BK_VBAR (1 << 1)
+#define RE_NO_BK_VBAR (1L << 1)
 
 /* If this bit is not set, plain + or ? serves as an operator, and \+, \? are 
      literals.
    If set, \+, \? are operators and plain +, ? are literals.  */
-#define RE_BK_PLUS_QM (1 << 2)
+#define RE_BK_PLUS_QM (1L << 2)
 
 /* If this bit is set, | binds tighter than ^ or $.
    If not set, the contrary.  */
-#define RE_TIGHT_VBAR (1 << 3)
+#define RE_TIGHT_VBAR (1L << 3)
 
 /* If this bit is set, then treat newline as an OR operator.
    If not set, treat it as a normal character.  */
-#define RE_NEWLINE_OR (1 << 4)
+#define RE_NEWLINE_OR (1L << 4)
 
 /* If this bit is set, then special characters may act as normal
    characters in some contexts. Specifically, this applies to:
@@ -76,71 +72,71 @@ extern int obscure_syntax;
    If this bit is not set, special characters (such as *, ^, and $)
    always have their special meaning regardless of the surrounding
    context.  */
-#define RE_CONTEXT_INDEP_OPS (1 << 5)
+#define RE_CONTEXT_INDEP_OPS (1L << 5)
 
 /* If this bit is not set, then \ before anything inside [ and ] is taken as 
      a real \.
    If set, then such a \ escapes the following character.  This is a
      special case for awk.  */
-#define RE_AWK_CLASS_HACK (1 << 6)
+#define RE_AWK_CLASS_HACK (1L << 6)
 
 /* If this bit is set, then \{ and \} or { and } serve as interval operators.
    If not set, then \{ and \} and { and } are treated as literals.  */
-#define RE_INTERVALS (1 << 7)
+#define RE_INTERVALS (1L << 7)
 
 /* If this bit is not set, then \{ and \} serve as interval operators and 
      { and } are literals.
    If set, then { and } serve as interval operators and \{ and \} are 
      literals.  */
-#define RE_NO_BK_CURLY_BRACES (1 << 8)
+#define RE_NO_BK_CURLY_BRACES (1L << 8)
 
 /* If this bit is set, then character classes are supported; they are:
      [:alpha:],	[:upper:], [:lower:],  [:digit:], [:alnum:], [:xdigit:],
      [:space:], [:print:], [:punct:], [:graph:], and [:cntrl:].
    If not set, then character classes are not supported.  */
-#define RE_CHAR_CLASSES (1 << 9)
+#define RE_CHAR_CLASSES (1L << 9)
 
 /* If this bit is set, then the dot re doesn't match a null byte.
    If not set, it does.  */
-#define RE_DOT_NOT_NULL (1 << 10)
+#define RE_DOT_NOT_NULL (1L << 10)
 
 /* If this bit is set, then [^...] doesn't match a newline.
    If not set, it does.  */
-#define RE_HAT_NOT_NEWLINE (1 << 11)
+#define RE_HAT_NOT_NEWLINE (1L << 11)
 
 /* If this bit is set, back references are recognized.
    If not set, they aren't.  */
-#define RE_NO_BK_REFS (1 << 12)
+#define RE_NO_BK_REFS (1L << 12)
 
 /* If this bit is set, back references must refer to a preceding
    subexpression.  If not set, a back reference to a nonexistent
    subexpression is treated as literal characters.  */
-#define RE_NO_EMPTY_BK_REF (1 << 13)
+#define RE_NO_EMPTY_BK_REF (1L << 13)
 
 /* If this bit is set, bracket expressions can't be empty.  
    If it is set, they can be empty.  */
-#define RE_NO_EMPTY_BRACKETS (1 << 14)
+#define RE_NO_EMPTY_BRACKETS (1L << 14)
 
 /* If this bit is set, then *, +, ? and { cannot be first in an re or
    immediately after a |, or a (.  Furthermore, a | cannot be first or
    last in an re, or immediately follow another | or a (.  Also, a ^
    cannot appear in a nonleading position and a $ cannot appear in a
    nontrailing position (outside of bracket expressions, that is).  */
-#define RE_CONTEXTUAL_INVALID_OPS (1 << 15)
+#define RE_CONTEXTUAL_INVALID_OPS (1L << 15)
 
 /* If this bit is set, then +, ? and | aren't recognized as operators.
    If it's not, they are.  */
-#define RE_LIMITED_OPS (1 << 16)
+#define RE_LIMITED_OPS (1L << 16)
 
 /* If this bit is set, then an ending range point has to collate higher
      or equal to the starting range point.
    If it's not set, then when the ending range point collates higher
      than the starting range point, the range is just considered empty.  */
-#define RE_NO_EMPTY_RANGES (1 << 17)
+#define RE_NO_EMPTY_RANGES (1L << 17)
 
 /* If this bit is set, then a hyphen (-) can't be an ending range point.
    If it isn't, then it can.  */
-#define RE_NO_HYPHEN_RANGE_END (1 << 18)
+#define RE_NO_HYPHEN_RANGE_END (1L << 18)
 
 
 /* Define combinations of bits for the standard possibilities.  */
@@ -229,10 +225,13 @@ extern int re_match (struct re_pattern_buffer *, char *, int, int,
 		     struct re_registers *);
 extern int re_match_2 (struct re_pattern_buffer *, char *, int,
 		       char *, int, int, struct re_registers *, int);
+extern long re_set_syntax (long syntax);
 
+#ifndef GAWK
 /* 4.2 bsd compatibility.  */
 extern char *re_comp (char *);
 extern int re_exec (char *);
+#endif
 
 #else /* !__STDC__ */
 
@@ -241,10 +240,13 @@ extern char *re_compile_pattern ();
 extern void re_compile_fastmap ();
 extern int re_search (), re_search_2 ();
 extern int re_match (), re_match_2 ();
+extern long re_set_syntax();
 
+#ifndef GAWK
 /* 4.2 bsd compatibility.  */
 extern char *re_comp ();
 extern int re_exec ();
+#endif
 
 #endif /* __STDC__ */
 
