@@ -10,8 +10,8 @@
  *
  * GAWK is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 1, or (at your option)
- * any later version.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * GAWK is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,7 +20,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with GAWK; see the file COPYING.  If not, write to
- * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ * the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 /*
@@ -43,6 +43,8 @@
 #define Get_Value(arg,buf,siz)	vmswork(Cli_Get_Value(arg,buf,siz))
 
 extern void   gawk_cmd();	/* created with $ SET COMMAND/OBJECT */
+#define GAWK_CMD ((const void *)gawk_cmd)
+extern void   _exit(int);
 static int    vms_usage(int);
 
 #define ARG_SIZ 250
@@ -77,7 +79,7 @@ vms_gawk()
 	   command, so we'll now attempt to generate a command from the
 	   foreign command string and parse that.
 	*/
-	sts = Cli_Parse_Command(gawk_cmd, COMMAND_NAME);
+	sts = Cli_Parse_Command(GAWK_CMD, COMMAND_NAME);
 	if (vmswork(sts))
 	    sts = Cli_Present("GAWK_P1");
     }
