@@ -10,8 +10,8 @@
  * 
  * GAWK is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 1, or (at your option)
- * any later version.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  * 
  * GAWK is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,10 +20,16 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with GAWK; see the file COPYING.  If not, write to
- * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ * the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include "awk.h"        /* really "../awk.h" */
+#define creat creat_dummy	/* one of gcc-vms's headers has bad prototype */
+#include "awk.h"
+#undef creat
+#include <fab.h>
+#ifndef O_RDONLY
+#include <fcntl.h>
+#endif
 #include <ssdef.h>
 #include <stsdef.h>
 
@@ -36,9 +42,9 @@
 # undef exit
 #endif
 void
-vms_exit( int errno )		/* note: local override of global 'errno' */
+vms_exit( int final_status )
 {
-    exit(errno == 0 ? SS$_NORMAL : (SS$_ABORT | STS$M_INHIB_MSG));
+    exit(final_status == 0 ? SS$_NORMAL : (SS$_ABORT | STS$M_INHIB_MSG));
 }
 #define exit(v) vms_exit(v)
 
@@ -55,7 +61,7 @@ vms_exit( int errno )		/* note: local override of global 'errno' */
 char *
 vms_strerror( int errnum )
 {
-    extern char *strerror( /* int, ... */ );
+    extern char *strerror P((int,...));
     return ( errnum != EVMSERR ? strerror(errnum)
 			       : strerror(EVMSERR, vaxc$errno) );
 }
@@ -92,22 +98,54 @@ unlink( const char *file_spec ) {
     return delete(tmp);
 }
 
+    /*
+     * Work-around an open(O_CREAT+O_TRUNC) bug (screwed up modification
+     * and creation dates when new version is created), and also use some
+     * VMS-specific file options.  Note:  optional 'prot' arg is completely
+     * ignored; gawk doesn't need it.
+     */
+#ifdef open
+# undef open
+#endif
+/* vms_open() - open a file, possibly creating it */
+int
+vms_open( const char *name, int mode, ... )
+{
+    extern int creat P((const char *,int,...));
+    extern int open  P((const char *,int,unsigned,...));
+
+    if (mode == (O_WRONLY|O_CREAT|O_TRUNC))
+	return creat(name, 0, "shr=nil", "mbc=24");
+    else {
+	struct stat stb;
+	const char *mbc, *shr = "shr=get";
+
+	if (stat(name, &stb) < 0) {	/* assume DECnet */
+	    mbc = "mbc=8";
+	} else {    /* ordinary file; allow full sharing iff record format */
+	    mbc = "mbc=12";
+	    if (stb.st_fab_rfm < FAB$C_STM) shr = "shr=get,put,upd";
+	}
+	return open(name, mode, 0, shr, mbc, "mbf=2");
+    }
+}
+
     /*
      * Check for attempt to (re-)open known file.
      */
 /* vms_devopen() - check for "SYS$INPUT" or "SYS$OUTPUT" or "SYS$ERROR" */
 int
-vms_devopen( const char *name )
+vms_devopen( const char *name, int mode )
 {
     FILE *file = NULL;
 
     if (strncasecmp(name, "SYS$", 4) == 0) {
 	name += 4;		/* skip "SYS$" */
-	if (strncasecmp(name, "INPUT", 5) == 0)
+	if (strncasecmp(name, "INPUT", 5) == 0 && (mode & O_WRONLY) == 0)
 	    file = stdin,  name += 5;
-	else if (strncasecmp(name, "OUTPUT", 6) == 0)
+	else if (strncasecmp(name, "OUTPUT", 6) == 0 && (mode & O_WRONLY) != 0)
 	    file = stdout,  name += 6;
-	else if (strncasecmp(name, "ERROR", 5) == 0)
+	else if (strncasecmp(name, "ERROR", 5) == 0 && (mode & O_WRONLY) != 0)
 	    file = stderr,  name += 5;
 	if (*name == ':')  name++;	/* treat trailing colon as optional */
     }
@@ -132,9 +170,9 @@ void tzset()
 # ifdef bcopy
 #  undef bcopy
 # endif
-void bcopy( char *src, char *dst, int len )
+void bcopy( const char *src, char *dst, int len )
 {
-    (void) OTS$MOVE3(len, src, dst);
+    (void) memcpy(dst, src, len);
 }
 #endif	/*!__GNUC__*/
 
